/*
 * Copyright (c) 2014 Samsung Electronics Co., Ltd All Rights Reserved
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef _MAPZEN_PLUGIN_H_
#define _MAPZEN_PLUGIN_H_

#include <maps_plugin.h>
#include <glib.h>

#ifdef __cplusplus
extern "C" {
#endif

typedef struct callback_geocode {
	maps_service_geocode_cb callback;
	void *data;
	int reqID;
} callback_info_geocode;

typedef struct callback_reverse_geocode {
	maps_service_reverse_geocode_cb callback;
	void *data;
	int reqID;
} callback_info_reverse_geocode;

typedef struct callback_route {
	maps_service_search_route_cb callback;
	void *data;
	int reqID;
} callback_info_route;

typedef struct callback_place {
	maps_service_search_place_cb callback;
	void *data;
	int reqID;
} callback_info_place;

typedef struct callback_place_list {
	maps_service_search_place_list_cb callback;
	void *data;
	int reqID;
} callback_info_place_list;

typedef struct callback_place_details {
	maps_service_get_place_details_cb callback;
	void *data;
	int reqID;
} callback_info_place_details;

/* TODO: IMPLEMENT THESE FOR REAL */

int maps_plugin_multi_reverse_geocode(const maps_coordinates_list_h geocode_list,
	const maps_preference_h preference, maps_service_multi_reverse_geocode_cb callback,
	void *user_data, int *request_id);

int maps_plugin_on_object(maps_view_h hView, const maps_view_object_h object, maps_view_object_operation_e operation);

/* ABOVE METHODS ARE THERE BUT RETURN  MAPS_ERROR_NOT_SUPPORTED */

int maps_plugin_create_map_view(maps_view_h hView, maps_plugin_map_view_ready_cb pCbFunc);

int maps_plugin_destroy_map_view(maps_view_h hView);

int maps_plugin_render_map(maps_view_h hView, const maps_coordinates_h coordinates, double zoom_factor, double rotation_angle);

int maps_plugin_move_center(maps_view_h hView, int delta_x, int delta_y);

int maps_plugin_set_scalebar(maps_view_h hView, bool enable);

int maps_plugin_get_scalebar(maps_view_h hView, bool *enabled);

int maps_plugin_screen_to_geography(maps_view_h hView, int x, int y, maps_coordinates_h *mapsCoord);

int maps_plugin_geography_to_screen(maps_view_h hView, const maps_coordinates_h mapsCoord, int* x, int* y);

int maps_plugin_get_min_zoom_level(maps_view_h hView, int *min_zoom_level);

int maps_plugin_get_max_zoom_level(maps_view_h hView, int *max_zoom_level);

int maps_plugin_get_center(maps_view_h hView, maps_coordinates_h *center);

int maps_plugin_capture_snapshot(maps_view_h view, void **data, int *width, int *height, maps_view_colorspace_type_e *cs);

int maps_plugin_init_module(maps_plugin_h *plugin, const char *module);

int maps_plugin_init(maps_plugin_h *plugin);

int maps_plugin_shutdown(maps_plugin_h plugin);

int maps_plugin_is_service_supported(maps_service_e service, bool *supported);

int maps_plugin_is_data_supported(maps_service_data_e data, bool *supported);

int maps_plugin_get_info(maps_plugin_info_h *info);

int maps_plugin_set_provider_key(const char *provider_key);

int maps_plugin_get_provider_key(char **provider_key);

int maps_plugin_set_preference(maps_item_hashtable_h preference);

int maps_plugin_get_preference(maps_item_hashtable_h *preference);

/* Geocode */
int maps_plugin_geocode(const char *address, const maps_preference_h preference, maps_service_geocode_cb callback, void *user_data, int *request_id);

/* Geocode inside area */
int maps_plugin_geocode_inside_area(const char *address, const maps_area_h bounds, const maps_preference_h preference, maps_service_geocode_cb callback, void *user_data, int *request_id);

/* Geocode by structured address */
int maps_plugin_geocode_by_structured_address(const maps_address_h address, const maps_preference_h preference, maps_service_geocode_cb callback, void *user_data, int *request_id);

/* Rev Geocode */
int maps_plugin_reverse_geocode(double latitude, double longitude, const maps_preference_h preference, maps_service_reverse_geocode_cb callback, void *user_data, int *request_id);

/* Route search */
int maps_plugin_search_route(const maps_coordinates_h origin, const maps_coordinates_h destination, maps_preference_h preference, maps_service_search_route_cb callback, void *user_data, int *request_id);

/* Route search waypoints */
int maps_plugin_search_route_waypoints(const maps_coordinates_h *waypoint_list, int waypoint_num, maps_preference_h preference, maps_service_search_route_cb callback, void *user_data, int *request_id);

/* Place search */
int maps_plugin_search_place(const maps_coordinates_h position, int distance, const maps_place_filter_h filter, maps_preference_h preference, maps_service_search_place_cb callback, void *user_data, int *request_id);

/* Place search by area */
int maps_plugin_search_place_by_area(const maps_area_h boundary, const maps_place_filter_h filter, maps_preference_h preference, maps_service_search_place_cb callback, void *user_data, int *request_id);

/* Place search by area, return list*/
int maps_plugin_search_place_list(const maps_area_h boundary, const maps_place_filter_h filter, maps_preference_h preference, maps_service_search_place_list_cb callback, void *user_data, int *request_id);

/* Get a Place via a uri */
int maps_plugin_get_place_details(const char* uri, maps_service_get_place_details_cb callback, void* user_data, int* request_id);

/* Place search by address */
int maps_plugin_search_place_by_address(const char *address, const maps_area_h boundary, const maps_place_filter_h filter, maps_preference_h preference, maps_service_search_place_cb callback, void *user_data, int *request_id);

/* Cancel Request */
int maps_plugin_cancel_request(int request_id);

#ifdef __cplusplus
}
#endif
#endif				/* _MAPZEN_PLUGIN_H_ */
