/* Copyright (c) 2014 Samsung Electronics Co., Ltd All Rights Reserved
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef __MAPZEN_PLUGIN_INTERNAL_H__
#define __MAPZEN_PLUGIN_INTERNAL_H__

#include <dlog.h>

#ifdef LOG_TAG
#undef LOG_TAG
#endif
#define LOG_TAG "MAPZEN_PLUGIN"

/*
 * Internal Macros
 */
#define MAPS_LOGD(fmt, args...)  LOGD(fmt, ##args)
#define MAPS_LOGW(fmt, args...)  LOGW(fmt, ##args)
#define MAPS_LOGI(fmt, args...)  LOGI(fmt, ##args)
#define MAPS_LOGE(fmt, args...)  LOGE(fmt, ##args)
#define MAPS_SECLOG(fmt, args...)  SECURE_LOGD(fmt, ##args)

#define MAPS_CHECK_CONDITION(condition, error, msg)	\
	do { \
		if (condition) { \
		} else { \
			MAPS_LOGE("%s(0x%08x)", msg, error); \
			return error; \
		} \
	} while (0)

#define MAPS_NULL_ARG_CHECK_RETURN_FALSE(arg)\
	do { \
		if (arg != NULL) { \
		} else	{ \
			MAPS_LOGE("MAPS_ERROR_INVALID_PARAMETER");  \
			return false; };	\
	} while (0)

#define MAPS_NULL_ARG_CHECK(arg)	\
	MAPS_CHECK_CONDITION(arg != NULL, MAPS_ERROR_INVALID_PARAMETER, "MAPS_ERROR_INVALID_PARAMETER")

#define MAPS_PRINT_ERROR_CODE_RETURN(code) \
	do { \
		MAPS_LOGE("%s(0x%08x)", #code, code); \
		return code;	\
	} while (0)

#endif /* __MAPZEN_PLUGIN_INTERNAL_H__ */
