/*
 * Copyright (c) 2014 Samsung Electronics Co., Ltd All Rights Reserved
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef _MAPZEN_API_H_
#define _MAPZEN_API_H_

#include "mapzen_types.h"
#include <maps_plugin.h>
#include <tizen_type.h>

/**
 * @ingroup	MAPZEN_ENGINE_MODULE
 * @defgroup	MAPZEN_ENGINE_API_MODULE
 *
 * @file mapzen_api.h
 * @brief This file contains the Mapzen engine API's that should be called by the plugin
 *
 * @addtogroup MAPZEN_ENGINE_MODULE
 * @{
 * @brief This provides APIs related to Mapzen engine.
 */

#ifdef __cplusplus
extern "C" {
#endif

  /* Callbacks */
typedef void (*mapzen_geocode_cb) (mapzen_error_e result, int request_id, GList *co_ordinates, void *user_data);

typedef void (*mapzen_reverse_geocode_cb) (mapzen_error_e result, int request_id, mapzen_address_resp_s *address, void *user_data);

typedef void (*mapzen_place_search_cb) (mapzen_error_e result, int request_id, GList *places, void *user_data);

typedef void (*mapzen_get_place_details_cb) (mapzen_error_e result, int request_id, GList *places, void *user_data);

typedef void (*mapzen_place_list_search_cb) (mapzen_error_e result, int request_id, GList *places, void *user_data);

typedef void (*mapzen_route_cb) (mapzen_error_e result, int request_id, mapzen_route_resp_s *route_info, void *user_data);

typedef void (*mapzen_maptile_cb) (mapzen_error_e result, int request_id, char *buffer, void *user_data);

int mapzen_init();

int mapzen_shutdown();

int mapzen_geocode(mapzen_geocode_req_s *req_details, mapzen_geocode_cb callback, int request_id, void *user_data);

int mapzen_cancel_request(int request_id);

int mapzen_reverse_geocode(mapzen_revgeocode_req_s *req_details, mapzen_reverse_geocode_cb callback, int request_id, void *user_data);

int mapzen_search_place(mapzen_search_req_s *req_details, mapzen_place_search_cb callback, int request_id, void *user_data);

int mapzen_get_place_details(mapzen_get_details_req_s *req_details, mapzen_get_place_details_cb callback, int request_id, void *user_data);

int mapzen_search_place_list(mapzen_search_req_s *req_details, mapzen_place_list_search_cb callback, int request_id, void *user_data);

int mapzen_start_route(mapzen_route_req_s *req_details, mapzen_route_cb callback, int request_id, void *user_data);

int mapzen_create_map_view(maps_view_h hView, maps_plugin_map_view_ready_cb pCbFunc, const char* providerKey);

int mapzen_destroy_map_view(maps_view_h hView);

int mapzen_render_map(maps_view_h hView, const maps_coordinates_h coordinates, double zoom_factor, double rotation_angle);

int mapzen_move_center(maps_view_h hView, int delta_x, int delta_y);

int mapzen_set_scalebar(maps_view_h hView, bool enable);

int mapzen_get_scalebar(maps_view_h hView, bool *enabled);

int mapzen_on_view_object(maps_view_h hView, const maps_view_object_h object, maps_view_object_operation_e operation);

int mapzen_screen_to_geography(maps_view_h hView, int x, int y, maps_coordinates_h *mapsCoord);

int mapzen_geography_to_screen(maps_view_h hView, const maps_coordinates_h mapsCoord, int* x, int* y);

int mapzen_get_min_zoom_level(maps_view_h hView, int *min_zoom_level);

int mapzen_get_max_zoom_level(maps_view_h hView, int *max_zoom_level);

int mapzen_get_center(maps_view_h hView, maps_coordinates_h *center);

int mapzen_capture_snapshot(maps_view_h hView, void **data, int *width, int *height, maps_view_colorspace_type_e *cs);

#ifdef __cplusplus
}
#endif

#endif				/* _MAPZEN_API_H_ */
