/*
 * Copyright (c) 2014 Samsung Electronics Co., Ltd All Rights Reserved
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include <stdio.h>
#include <glib.h>
#include <pthread.h>
#include <curl/curl.h>

#include "mapzen_geocode.h"
#include "mapzen_types.h"
#include "mapzen_server_private.h"
#include "mapzen_debug.h"
#include "mapzen_queue.h"
#include "mapzen_restcurl.h"
#include "mapzen_util.h"

#define GEOCODE_URL	"https://search.mapzen.com/v1/search?api_key=%s"

int query_geocode_within_bounding_box(gchar *maps_key, char *address, coords_s top_left, coords_s bottom_right, int num_results, gpointer user_data)
{
	char url[1024] = "";
	char tmpStr[512] = "";

	if (maps_key != NULL)
		snprintf(tmpStr, sizeof(tmpStr), GEOCODE_URL, maps_key);
	else
		snprintf(tmpStr, sizeof(tmpStr), GEOCODE_URL, "null");

	STRCPY(url, tmpStr);
	snprintf(tmpStr, sizeof(tmpStr), "&boundary.rect.min_lat=%f&boundary.rect.min_lon=%f&boundary.rect.max_lat=%f&boundary.rect.max_lon=%f",
			bottom_right.latitude, top_left.longitude, top_left.latitude, bottom_right.longitude);
	STRCAT(url, tmpStr);

	snprintf(tmpStr, sizeof(tmpStr), "&size=%d", num_results);
	STRCAT(url, tmpStr);

	char *encoded = curl_escape(address, 0);
	snprintf(tmpStr, sizeof(tmpStr), "&text=%s", encoded);
	STRCAT(url, tmpStr);
	curl_free(encoded);

	add_handle(url, REQ_TYPE_GEOCODE, user_data);

	return 0;
}

int query_geocode_within_circle(gchar *maps_key, char *address, coords_s center, double radius, int num_results, gpointer user_data)
{
	char url[1024] = "";
	char tmpStr[512] = "";

	if (maps_key != NULL)
		snprintf(tmpStr, sizeof(tmpStr), GEOCODE_URL, maps_key);
	else
		snprintf(tmpStr, sizeof(tmpStr), GEOCODE_URL, "null");

	STRCPY(url, tmpStr);

	snprintf(tmpStr, sizeof(tmpStr), "&boundary.circle.lon=%f&boundary.circle.lat=%f&boundary.circle.radius=%f",
			center.longitude, center.latitude, radius);
	STRCAT(url, tmpStr);

	snprintf(tmpStr, sizeof(tmpStr), "&size=%d", num_results);
	STRCAT(url, tmpStr);

	char *encoded = curl_escape(address, 0);
	snprintf(tmpStr, sizeof(tmpStr), "&text=%s", encoded);
	STRCAT(url, tmpStr);
	curl_free(encoded);

	add_handle(url, REQ_TYPE_GEOCODE, user_data);

	return 0;
}

int query_geocode(gchar *maps_key, char *address, int num_results, gpointer user_data)
{
	MAP_DEBUG("geocode address : %s", address);
	char url[1024] = "";
	char tmpStr[512] = "";

	if (maps_key != NULL)
		snprintf(tmpStr, sizeof(tmpStr), GEOCODE_URL, maps_key);
	else
		snprintf(tmpStr, sizeof(tmpStr), GEOCODE_URL, "null");

	STRCPY(url, tmpStr);

	snprintf(tmpStr, sizeof(tmpStr), "&size=%d", num_results);
	STRCAT(url, tmpStr);

	char *encoded = curl_escape(address, 0);
	snprintf(tmpStr, sizeof(tmpStr), "&text=%s", encoded);
	STRCAT(url, tmpStr);
	curl_free(encoded);

	add_handle(url, REQ_TYPE_GEOCODE, user_data);

	return 0;
}
