/*
 * Copyright (c) 2014 Samsung Electronics Co., Ltd All Rights Reserved
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include <stdio.h>
#include <stdlib.h>
#include <glib.h>
#include <pthread.h>
#include <curl/curl.h>

#include "mapzen_place.h"
#include "mapzen_types.h"
#include "mapzen_debug.h"
#include "mapzen_queue.h"
#include "mapzen_restcurl.h"
#include "mapzen_util.h"

#define CATEGORIES_URL	"https://search.mapzen.com/v1/nearby?api_key=%s"
#define KEYWORD_CATEGORIES_URL	"https://search.mapzen.com/v1/search?api_key=%s"
#define PLACE_DETAILS_URL	"https://search.mapzen.com/v1/place?api_key=%s"

int query_places(gchar *maps_key, gchar *search_str, gchar *categories, mapzen_boundary_s *boundary, int num_res, gpointer user_data, mapzen_req_type type)
{
	char url[1024] = "";
	char tmpStr[512] = "";

	if (search_str != NULL || boundary->type == MAPZEN_BOUNDARY_RECT) {
		if (maps_key != NULL)
			snprintf(tmpStr, sizeof(tmpStr), KEYWORD_CATEGORIES_URL, maps_key);
		else snprintf(tmpStr, sizeof(tmpStr), KEYWORD_CATEGORIES_URL, "null");
		STRCPY(url, tmpStr);

		if (search_str != NULL) {
			char *encoded = curl_escape(search_str, 0);
			snprintf(tmpStr, sizeof(tmpStr), "&text=%s", encoded);
			STRCAT(url, tmpStr);
			curl_free(encoded);
		}

		if (categories != NULL) {
			char *encoded = curl_escape(categories, 0);
			snprintf(tmpStr, sizeof(tmpStr), "&categories=%s", encoded);
			STRCAT(url, tmpStr);
			curl_free(encoded);
		}
	} else {
		if (maps_key != NULL)
			snprintf(tmpStr, sizeof(tmpStr), CATEGORIES_URL, maps_key);
		else snprintf(tmpStr, sizeof(tmpStr), CATEGORIES_URL, "null");
		STRCPY(url, tmpStr);

		if (categories != NULL) {
			char *encoded = curl_escape(categories, 0);
			snprintf(tmpStr, sizeof(tmpStr), "&categories=%s", encoded);
			STRCAT(url, tmpStr);
			curl_free(encoded);
		}
	}

	if (boundary != NULL) {
		if (boundary->type == MAPZEN_BOUNDARY_RECT) {
			snprintf(tmpStr, sizeof(tmpStr), "&boundary.rect.min_lat=%f&boundary.rect.min_lon=%f&boundary.rect.max_lat=%f&boundary.rect.max_lon=%f",
					boundary->rect.bottom_right.latitude, boundary->rect.top_left.longitude,
					boundary->rect.top_left.latitude, boundary->rect.bottom_right.longitude);
			STRCAT(url, tmpStr);
		} else if (boundary->type == MAPZEN_BOUNDARY_CIRCLE) {

			if (search_str == NULL) {
				snprintf(tmpStr, sizeof(tmpStr), "&point.lon=%f&point.lat=%f&boundary.circle.radius=%f",
						boundary->circle.center.longitude, boundary->circle.center.latitude, boundary->circle.radius);
				STRCAT(url, tmpStr);
			} else {
				snprintf(tmpStr, sizeof(tmpStr), "&boundary.circle.lon=%f&boundary.circle.lat=%f&boundary.circle.radius=%f",
						boundary->circle.center.longitude, boundary->circle.center.latitude, boundary->circle.radius);
				STRCAT(url, tmpStr);
			}
		}
	}

	snprintf(tmpStr, sizeof(tmpStr), "&size=%d", num_res);
	STRCAT(url, tmpStr);

	add_handle(url, type, user_data);

	return 0;
}

int query_place_details(gchar *maps_key, gchar *ids, gpointer user_data)
{
	char url[1024] = "";
	char tmpStr[512] = "";

	if (maps_key != NULL)
		snprintf(tmpStr, sizeof(tmpStr), PLACE_DETAILS_URL, maps_key);
	else
		snprintf(tmpStr, sizeof(tmpStr), PLACE_DETAILS_URL, "null");
	STRCPY(url, tmpStr);

	if (ids != NULL) {
		char *encoded = curl_escape(ids, 0);
		snprintf(tmpStr, sizeof(tmpStr), "&ids=%s", encoded);
		STRCAT(url, tmpStr);
		curl_free(encoded);
	}

	add_handle(url, REQ_TYPE_PLACES_DETAILS, user_data);

	return 0;
}
