/*
 * Copyright (c) 2014 Samsung Electronics Co., Ltd All Rights Reserved
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef _MAPZEN_QUEUE_H_
#define _MAPZEN_QUEUE_H_

#include <glib.h>
#include "mapzen_server_private.h"

typedef struct {
	mapzen_req_type type;
	void *request;
} mapzen_request_s;

typedef struct {
	mapzen_geocode_req_s *req_details;
	int requestId;
	mapzen_geocode_cb geocode_cb;
	void *user_data;
} mapzen_geocode_req;

typedef struct {
	mapzen_revgeocode_req_s *req_details;
	int requestId;
	mapzen_reverse_geocode_cb revgeocode_cb;
	void *user_data;
} mapzen_revgeocode_req;

typedef struct {
	mapzen_route_req_s *req_details;
	mapzen_route_cb route_cb;
	int requestId;
	void *user_data;
} mapzen_route_req;

typedef struct {
	mapzen_search_req_s *req_details;
	mapzen_place_search_cb search_place_cb;
	int requestId;
	void *user_data;
} mapzen_search_place_req;

typedef struct {
	mapzen_get_details_req_s *req_details;
	mapzen_get_place_details_cb get_place_details_cb;
	int requestId;
	void *user_data;
} mapzen_get_place_details_req;

typedef struct {
	mapzen_search_req_s *req_details;
	mapzen_place_list_search_cb search_place_list_cb;
	int requestId;
	void *user_data;
} mapzen_search_place_list_req;

int add_to_geocode_list(mapzen_geocode_req_s *req_details, mapzen_geocode_cb callback, int request_id, void *user_data);
int add_to_revgeocode_list(mapzen_revgeocode_req_s *req_details, mapzen_reverse_geocode_cb callback, int request_id, void *user_data);
int add_to_route_list(mapzen_route_req_s *req_details, mapzen_route_cb callback, int request_id, void *user_data);
int add_to_place_list(mapzen_search_req_s *req_details, mapzen_place_search_cb callback, int request_id, void *user_data);
int add_to_place_details_list(mapzen_get_details_req_s *req_details, mapzen_get_place_details_cb callback, int request_id, void *user_data);
int add_to_places_list(mapzen_search_req_s *req_details, mapzen_place_list_search_cb callback, int request_id, void *user_data);

int remove_from_request_list(int request_id);

int start_geocode_service(mapzen_geocode_req_s *req_details, mapzen_geocode_cb callback, int request_id, void *user_data);
int start_reversegeocode_service(mapzen_revgeocode_req_s *req_details, mapzen_reverse_geocode_cb callback, int request_id, void *user_data);
int start_route_service(mapzen_route_req_s *req_details, mapzen_route_cb callback, int request_id, void *user_data);
int start_place_service(mapzen_search_req_s *req_details, mapzen_place_search_cb callback, int request_id, void *user_data);
int start_place_details_service(mapzen_get_details_req_s *req_details, mapzen_get_place_details_cb callback, int request_id, void *user_data);
int start_places_list_service(mapzen_search_req_s *req_details, mapzen_place_list_search_cb callback, int request_id, void *user_data);

int mapzen_init_queue();
int mapzen_deinit_queue();
int mapzen_push_to_queue(mapzen_resp_type type, gpointer data);

#endif /* _MAPZEN_QUEUE_H_ */
