/*
 * Copyright (c) 2014 Samsung Electronics Co., Ltd All Rights Reserved
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef _MAPZEN_PRIVATE_H_
#define _MAPZEN_PRIVATE_H_

#include "mapzen_api.hpp"

typedef enum {
	REQ_TYPE_GEOCODE = 0,
	REQ_TYPE_REVGEOCODE,
	REQ_TYPE_PLACES,
	REQ_TYPE_PLACES_LIST,
	REQ_TYPE_PLACES_DETAILS,
	REQ_TYPE_ROUTE,
	REQ_TYPE_NONE
} mapzen_req_type;

typedef enum {
	RESP_TYPE_GEOCODE = 0,
	RESP_TYPE_REVGEOCODE,
	RESP_TYPE_PLACES,
	RESP_TYPE_PLACES_LIST,
	RESP_TYPE_PLACES_DETAILS,
	RESP_TYPE_ROUTE,
	RESP_TYPE_NONE
} mapzen_resp_type;

typedef struct {
	int requestId;
	mapzen_geocode_cb geocode_cb;
	void *user_data;
} MapzenGeocodeQueryData;

typedef struct {
	int requestId;
	mapzen_reverse_geocode_cb reverse_geocode_cb;
	void *user_data;
} MapzenRevGeocodeQueryData;

typedef struct {
	int requestId;
	mapzen_route_cb route_cb;
	coords_s origin;
	coords_s destination;
	route_unit unit;
	route_type type;
	void *user_data;
} MapzenRouteQueryData;

typedef struct {
	int requestId;
	mapzen_place_search_cb place_search_cb;
	void *user_data;
} MapzenPlaceQueryData;

typedef struct {
	int requestId;
	mapzen_place_list_search_cb place_list_search_cb;
	void *user_data;
} MapzenPlaceListQueryData;

typedef struct {
	int requestId;
	mapzen_get_place_details_cb get_place_details_cb;
	void *user_data;
} MapzenPlaceDetailsQueryData;

typedef struct {
	mapzen_error_e error;
	int requestId;
	mapzen_geocode_cb geocode_cb;
	GList *coords;
	void *user_data;
} MapzenGeocodeResponseData;

typedef struct {
	mapzen_error_e error;
	int requestId;
	mapzen_reverse_geocode_cb reverse_geocode_cb;
	mapzen_address_resp_s *addressDetails;
	void *user_data;
} MapzenRevGeocodeResponseData;

typedef struct {
	mapzen_error_e error;
	int requestId;
	mapzen_route_cb route_cb;
	mapzen_route_resp_s *routeResponse;
	void *user_data;
} MapzenRouteResponseData;

typedef struct {
	mapzen_error_e error;
	int requestId;
	mapzen_place_search_cb place_search_cb;
	GList *places;
	void *user_data;
} MapzenPlaceResponseData;

typedef struct {
	mapzen_error_e error;
	int requestId;
	mapzen_get_place_details_cb get_place_details_cb;
	GList *places;
	void *user_data;
} MapzenPlaceDetailsResponseData;

typedef struct {
	mapzen_error_e error;
	int requestId;
	mapzen_place_list_search_cb place_list_search_cb;
	GList *places;
	void *user_data;
} MapzenPlaceListResponseData;

typedef struct {
	mapzen_resp_type type;
	void *data;
} MapzenQueueData;

#endif /* _MAPZEN_PRIVATE_H_ */
